/********************************************************************************************************
class AddFileToCategoryDlg derived from wxDialog
parent Class wxDialog
Files: AddFileToCategoryDlg.cpp, AddFileToCategoryDlg.h
Authors: Thalia Lewis and Muhammad Zulfigar
JPC36 Wx-View
Function definitions of class AddFileToCategoryDlg
Last Updated: August 9, 2000


Members 								Purpose
wxString filename                       receives the name of the file to be added to the treectrl
wxStaticText * prompt;                  prompts the user to choose the category to which the file shoudl be added
CategoryTreeCtrl * tree;                treectrl containg Category objects derived from class wxTreeCtrl
wxButton * ok;                          adds file
wxButton * cancel;                      cancels file addition
*********************************************************************************************************/

#include "AddFileToCategoryDlg.h"

#include "wx-view.h"
#include "category_treectrl.h"

#include <algorithm>
using namespace std;

/******************************************************************************************************
	Function: AddFileToCategoryDlg::AddFileToCategoryDlg
	Receives: N/A constructor
	Returns:  N/A constructor
	Description: Configures dialog containing a CategoryTreeCtrl allowing user to add a file to a Category
*******************************************************************************************************/
AddFileToCategoryDlg::AddFileToCategoryDlg (wxWindow * parent, const wxString & title, const wxString & filename)
	    : wxDialog (parent, -1, title, wxDefaultPosition, wxSize(300,200)), filename(filename)
{
		
	//create dialogbox with CategoryTreeCtrl and OK and Cancel buttons	
	wxString caption = translation["Add file "] + filename + translation[" to category:"];
   	prompt = new wxStaticText (this, -1, caption, wxPoint(1, 2*MARGIN), wxDefaultSize);
   	prompt->CentreOnParent(wxHORIZONTAL);
		
   	tree = new CategoryTreeCtrl (this, -1);
   	tree->SetSize (1, prompt->GetRect().GetBottom() + MARGIN, 270, 150);
   	tree->Expand(tree->GetRootItem());
		
    int cli_w, cli_h;
    GetClientSize (&cli_w, &cli_h);
    int w, h;
    GetSize (&w, &h);
			
   	ok = new wxButton (this, wxID_OK, translation["OK"],
   	                   wxPoint (1, tree->GetRect().GetBottom() + 2*MARGIN));
		
   	cancel = new wxButton (this, wxID_CANCEL, translation["Cancel"],
   							wxPoint (1, ok->GetRect().GetTop()));
				
    const int dx = w - cli_w;
    const int dy = h - cli_h;
	
    const int new_width = dx + max(prompt->GetRect().GetWidth(), tree->GetRect().GetWidth()) + 4*MARGIN;
	const int new_height = dy + ok->GetRect().GetBottom() + 2*MARGIN;
		
   	SetSize (new_width, new_height);
		
   	prompt->CentreOnParent(wxHORIZONTAL);
   	tree->CentreOnParent(wxHORIZONTAL);
		
   	ok->Move (tree->GetRect().GetLeft(), ok->GetRect().GetTop());
   	cancel->Move (tree->GetRect().GetRight() - ok->GetRect().GetWidth(),
   	              cancel->GetRect().GetTop());
		
   	CentreOnParent ();
}


/******************************************************************************************************
	Function: AddFileToCategoryDlg::ShowModal()
	Receives: void
	Returns:  int (wxID_OK, wxID_CANCEL)
	Description: Dialog box prompts user to add a file to CategoryTreeCtrl
*******************************************************************************************************/
int AddFileToCategoryDlg::ShowModal ()
{
	int reply;
	
	bool added_to_category;
	
	do
	{
		reply = wxDialog::ShowModal();
		added_to_category = true;
		
		if (reply == wxID_OK)
		{
			added_to_category = tree->add_file (this, filename, tree->GetSelection());
		}
		
	} while (!added_to_category );
		
	return reply;
}

//*******************************************************************************************************



