/******************************************
Title: PreviewPanel.Cpp
Mike MacIntyre, David Boys
Function definitions for Class PreviewPanel
last updated : August 9, 2000
******************************************/

#include "PreviewPanel.h"

#include <algorithm>
using std::min;

///// FUNCTION DEFINITIONS /////

// *** PreviewPanel Class ***

/**********************************************************************
   Function: PreviewPanel::PreviewPanel
   Receives: wxWindow *, const wxPoint &, const wxSize &
   Returns:  N/A (Constructor)
   Description: Constructs a derived wxPanel that conatains a wxStaticbitmap
   				to place the preview image on
**********************************************************************/
PreviewPanel::PreviewPanel (wxWindow * parent, const wxPoint & pos, const wxSize & size)
		: wxPanel (parent, -1, pos, size, wxSUNKEN_BORDER)
{
	preview = new wxStaticBitmap (this, -1, wxBitmap(), wxDefaultPosition, wxDefaultSize);
	preview->Show(false);
}

/**********************************************************************
   Function: PreviewPanel::~PreviewPanel
   Receives: N/A (Destructor)
   Returns:  N/A (Destructor)
   Description: Deallocates memory
**********************************************************************/
PreviewPanel::~PreviewPanel()
{
	delete (preview);
}

/**********************************************************************
   Function: PreviewPanel::preview_iamge
   Receives: wxImage *
   Returns:  void
   Description: Receives an image pointer, to then re-scale to the size
   				of the PreviewPanel, and displays the re-scaled image
**********************************************************************/
void PreviewPanel::preview_image(wxImage * img)
{
	clear_background();
    const int w = img->GetWidth();
    const int h = img->GetHeight();

    int max_w, max_h;
    GetClientSize (&max_w, &max_h);

    const double ratio_w = static_cast<double>(max_w) / w;
    const double ratio_h = static_cast<double>(max_h) / h;
    const double ratio = min(ratio_w, ratio_h);

    if (ratio < 1)
    {
    	img->Rescale (w * ratio, h * ratio);
    }

    preview->SetBitmap (img->ConvertToBitmap());
    Clear();
    preview->CentreOnParent();
    preview->Show(true);
}

/**********************************************************************
   Function: PreviewPanel::clear_preview
   Receives: N/A (no parameters)
   Returns:  void
   Description: Clears the PreviewPanel preview image
**********************************************************************/
void PreviewPanel::clear_preview()
{
	preview->SetBitmap (wxBitmap());
	preview->Show(false);
	clear_background();
}

/**********************************************************************
   Function: PreviewPanel::clear_background
   Receives: N/A (no parameters)
   Returns:  void
   Description: Clears the background of the PreviewPanel
**********************************************************************/
void PreviewPanel::clear_background()
{
	int w, h;
	GetClientSize (&w, &h);
		
	wxClientDC dc(this);
	dc.SetPen (*wxTRANSPARENT_PEN);
	dc.SetBrush (wxBrush(GetBackgroundColour(), wxSOLID));
	dc.DrawRectangle (0, 0, w, h);
}

