///////////////////////////////////////////////////////////////////////////////////////////
// Authors: Eva Bunodiere, Yasmin Kossak
// ZoomSettingsDialog.cpp
// ZoomSettingsDialog Class Function Definitions
// JPC36 Wx-View 
// July 25th 2000
// Technical Advisor : Carlos Moreno
///////////////////////////////////////////////////////////////////////////////////////////

#include "SetIntervalDialog.h"

#include "wx-view.h"

#include <cmath>
#include <sstream>
using namespace std;

/******************************************************************************************
	Function: SetIntervalDialog::SetIntervalDialog
	Receives: N/A (constructor)
	Returns: N/A (constructor)
	Description: 
		Calls the wxDialog constructor to create a new set interval dialog for the application.
		Creates a wxTextCtrl for the interval.
		Resizes the Dialog after placing the controls to ensure correct positioning. Adds Cancel
		and OK buttons to dismiss the dialog.
*******************************************************************************************/
SetIntervalDialog::SetIntervalDialog(wxWindow * parent, wxWindowID id, const wxString &title, double saved_interval)
    : wxDialog (parent, id, title, wxDefaultPosition, wxSize(275, 225))
{
    const int margin = 20;

	// Place all controls in positions relative to the previously created controls
    
    saved_interval /= 1000;
    ostringstream buffer;
    buffer << saved_interval;
    string temp = buffer.str();

	set_interval_label = new wxStaticText(this, -1, translation["Enter_interval"], wxPoint(margin, margin));
	const int y_text = set_interval_label->GetRect().GetBottom() + 5;
	set_interval_text = new wxTextCtrl (this, -1, "", wxPoint(margin, y_text));
	set_interval_text->SetValue(temp.c_str());
	
	const int x_ok = set_interval_label->GetRect().GetRight() + 20;
    ok = new wxButton(this, wxID_OK, translation["OK"], wxPoint(x_ok, margin));
    const int y_cancel = ok->GetRect().GetBottom() + 15;
	cancel = new wxButton(this, wxID_CANCEL, translation["Cancel"], wxPoint(x_ok, y_cancel));
	
	int w, h, client_w, client_h;
	GetSize (&w, &h);
	GetClientSize (&client_w, &client_h);
	
	const int dx = w - client_w;
	const int dy = h - client_h;
	
	const int width = ok->GetRect().GetRight() + margin + dx;
	const int height = cancel->GetRect().GetBottom() + margin + dy;
	
	SetSize (width, height);
	CentreOnParent(wxBOTH);
}

/******************************************************************************************
	Function: SetInterval::read_interval()
	Receives: N/A
	Returns: double
	Description:
		Returns the input of the user from the text control and converts the result to a
		double.
*******************************************************************************************/
double SetIntervalDialog::read_interval() const
{
    return atof(set_interval_text->GetValue().c_str()) * 1000;
}

