/**********************************************************************************************************
class CatDialog derived from wxDialog
parent class: ImageDialog
Authors: Thalia Lewis and Muhammad Zulfigar
Files:: catdialog.h, catedialog.cpp
Function definitions of Class CatDialog
Last updated: August 9, 2000

No Members
***********************************************************************************************************/

#include "catdialog.h"

#include "imagedialog.h"
#include "category_treectrl.h"
#include "category.h"

#include <wx/treectrl.h>
#include <wx/string.h>
#include "wx-view.h"

#include <vector>
using namespace std;

//----------------------------------------------------------------------------------------------------------

BEGIN_EVENT_TABLE(CatDialog, ImageDialog)	
    EVT_TREE_SEL_CHANGED (ID_category_tree, CatDialog::OnItemActivated)
    EVT_BUTTON           (ID_delete_file, CatDialog::OnDeleteFile)
END_EVENT_TABLE()


/********************************************************************************************
	Function: CatDialog::CatDialog
	Receives: N/A constructor
	Returns:  N/A constructor
	Description:
		Configures Dialog box that allows user to browse image_files	
*********************************************************************************************/
CatDialog::CatDialog(wxWindow* parent, wxWindowID id, const wxString& title, const wxPoint& pos,
			const wxSize& size, const wxString & map_filename, const wxString & config_dir, int icon_size)
            :ImageDialog(parent, id, title, pos, size, map_filename, config_dir, icon_size)
{
	tree = new CategoryTreeCtrl(this, ID_category_tree);
	set_tree_size_and_position();
    tree->Expand(tree->GetRootItem());
    wxButton * delete_button;

    int w, h;
	GetClientSize(&w, &h);

    delete_button = new wxButton(this, ID_delete_file, translation[" Remove Image From Category "]);	

	int but_w, but_h;
	
	delete_button -> GetSize(&but_w, &but_h);	
	
	delete_button -> Move(w - (but_w + BORDER)-100, h - (2 * (but_h + BORDER)));

    delete_button->Show(true);

}

/********************************************************************************************
	Function: CatDialog::display_image_icons
	Receives: const vector<wxString> & image_names
	Returns: void
	Description: Calls parent(ImageDialog) function that places images on the dialog image panel.
********************************************************************************************/
void CatDialog::display_image_icons(const vector<wxString> & image_names)
{	
	for (int i = 0; i < image_names.size(); i++)
	{	 	
	 	add_to_panel_creation_files(image_names[i]);
	}

	reverse_panel_creation_files();
	start_panel_timer();
}


/********************************************************************************************
	Function: CatDialog::get_image_icons
	Receives: vector <wxString> & image_names, const wxTreeItemId & selected_id
	Returns: void
	Description: Calls treectrl function which get names of files stored in a treectrl category
*********************************************************************************************/

void CatDialog::get_image_icons(vector <wxString> & image_names, const wxTreeItemId & selected_id)
{	
	dynamic_cast<CategoryTreeCtrl *>(tree)->get_filenames(image_names, selected_id);
}	


/********************************************************************************************
	Function: CatDialog::refresh_image_icons
	Receives: const vector<wxString> & image_names
	Returns: void
	Description: Updates images on the dialog image panel (if files have been removed from treectrl category).
********************************************************************************************/

void CatDialog::refresh_image_icons(const wxTreeItemId & selected_id)
{

    vector <wxString> image_names;

	// clear_preview();  (already in begin_panel_creation)
	
	// clear_panels();  (already in begin_panel_creation)
	
	get_image_icons(image_names, selected_id);
	
	display_image_icons(image_names);
}






//--------------------------------------------------------------------------------------------------------
// Event Handlers

/********************************************************************************************
	Function: CatDialog::OnItemActivated
	Receives: wxTreeEvent & event
	Returns: void
	Description: When user selects a category, if the category contains image files, they are
				displayed on an image panel
*********************************************************************************************/

void CatDialog::OnItemActivated(wxTreeEvent & event)
{		
	begin_panel_creation();
	refresh_image_icons(event.GetItem());
}



/********************************************************************************************
	Function: CatDialog::OnDeleteFile
	Receives: wxCommandEvent & event
	Returns: void
	Description: Calls function which removes an image file from a treectrl category.
********************************************************************************************/

void CatDialog::OnDeleteFile(wxCommandEvent &)
{ 	
 	vector <wxString> file_names;
 	
 	file_names = get_temp_selections(); 	
 	
    if (!file_names.empty())
    {
 		for (int i = 0; i < file_names.size(); i++)
 		{
 			dynamic_cast<CategoryTreeCtrl *>(tree)->remove_file(this, file_names[i]);
 		}
 		
        begin_panel_creation();
 		refresh_image_icons(tree->GetSelection());	
 	} 			
}


/*********************************************	End catdialog.cppp ****************************************/