/**********************************************************************************************************
class CategoryTreeCtrl derived from wxTreeCtrl
parent class: wxTreeCtrl
Authors: Thalia Lewis and Muhammad Zulfigar
Files:: category_treectrl.h, category_treectrl.cpp
class definition of Class CategoryTreeCtrl
Last updated: August 9, 2000

Member                                 Purpose
Category * root_category_ptr; 		Points to the "root" category owned by the treectrl (ie, the category that contains all sub-categories on the tree)	
wxTreeItemId root_id; 				Stores the wxTreeItemId of the "root" category		
wxMenu * menu;					    Pop-up menu for Adding, Modifying, or Removing categories
wxPoint last_click_position;		Treectrl stores position of the last left-click in order to place the pop-up menu
	

***********************************************************************************************************/

#ifndef __CATTREE_CTRL__H_
#define __CATTREE_CTRL__H_


#include "wx/wxprec.h"

#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/log.h"
#include "wx/image.h"
#include "wx/treectrl.h"
#include <wx/string.h>
#include "wx/treectrl.h"

#include "category.h"

#include <string>
#include <vector>
using namespace std;

enum
{
    ID_add_subcategory = 1,
    ID_modify_category,
	ID_remove_category
};

class CategoryTreeCtrl: public wxTreeCtrl
{
public:

	CategoryTreeCtrl(wxWindow *parent, const wxWindowID id);
	~CategoryTreeCtrl()
	{}	         			

    void add_category(wxWindow * parent, const wxTreeItemId &selected_id);
    bool add_file(wxWindow * parent, const wxString & file_name, const wxTreeItemId &selected_id) const;
    void add_new_id(const wxTreeItemId & selected_id, const wxTreeItemId & new_id);
    void build_tree(const wxTreeItemId &parent_id, Category &category, bool first_entry = true);
    Category * get_category(const wxTreeItemId &selected_id);
    const Category * get_category(const wxTreeItemId & selected_id) const;
    void get_filenames(vector<wxString> &file_names, const wxTreeItemId & selected_id) const;
    const vector<string> get_category_file_contents()const;
    const Category * get_root_category()const;
    Category * get_root_category();
    void modify_category(wxWindow *parent, const wxTreeItemId &selected_id);
    void remove_category(wxWindow * parent, const wxTreeItemId & selected_id);
    void remove_file(wxWindow * parent, const wxString & file_name) const;
	void update_category_file_contents() const;
	
	void OnRightClick(wxMouseEvent &);
	void OnLeftClick(wxMouseEvent &);
	
	void OnMenuAddCategory (wxCommandEvent &);
	void OnMenuRemoveCategory (wxCommandEvent &);
	void OnMenuModifyCategory (wxCommandEvent &);

private:
	Category * root_category_ptr;

	wxTreeItemId root_id;
	
	wxMenu * menu;	                // pop-up menu for Adding, Modifying, or Removing categories
	
	wxPoint last_click_position;    // treectrl stores position of the last left-click
	                                // in order to place the pop-up menu (due to an apparent
	                                // limitation of the wxTreeCtrl -- missing BoundingRect method)

	DECLARE_EVENT_TABLE()
};

#endif
