/********************************
dirdialog.cpp
Mike MacIntyre, David Boys
Function definitions for Class DirDialog
last updated : August 7, 2000
*********************************/

#include "dirdialog.h"

BEGIN_EVENT_TABLE(DirDialog, ImageDialog)
    //EVT_TREE_ITEM_ACTIVATED (ID_tree, DirDialog::OnItemActivated)//DirDialog::Ignore)
    EVT_TREE_ITEM_EXPANDING (ID_tree, DirDialog::OnItemExpanding)
    EVT_TREE_SEL_CHANGED    (ID_tree, DirDialog::OnItemSelected)
    EVT_TEXT_ENTER(ID_enter_box, DirDialog::OnEnter)
    EVT_CHOICE(ID_filter, DirDialog::OnChoice)

END_EVENT_TABLE()


/**********************************************************************
   Function:  DirDialog::DirDialog
   Receives:  N/A (constructor)            
   Returns:   N/A (constructor)
   Description:
             Initializations of DirDialog object. 
**********************************************************************/


DirDialog::DirDialog(wxWindow* parent, wxWindowID id, const wxString& title, const wxPoint& pos,
            const wxSize& size, const wxString & map_filename, const wxString & config_dir, int icon_size)
            :ImageDialog(parent, id, title, pos, size, map_filename , config_dir, icon_size)
{
	tree = new DirTreeCtrl(this, ID_tree);
	set_tree_size_and_position();

    enable_filespec_controls();
	
    tree->Expand(tree->GetRootItem());
    tree->SetFocus();
}

/**********************************************************************
   Function: DirDialog::OnItemExpanding
   Receives: (wxTreeEvent& event)             
   Returns:  (void)
   Description:
             Adds the branches to the tree as it expands. Expanding the
             entire tree at once is too slow.
**********************************************************************/

void DirDialog::OnItemExpanding(wxTreeEvent& event)
{
	
    wxLogNull no_message;
	wxTreeItemId id = event.GetItem();
	wxString path = get_full_path(id);	
	wxString filename;
    wxDir current_dir(path);
	long cookie;
    wxTreeItemId child_id = tree -> GetFirstChild(id, cookie);

    wxBeginBusyCursor();
	bool cont = current_dir.GetFirst(&filename, wxEmptyString, wxDIR_DIRS | wxDIR_HIDDEN);

	while ( cont )
    {
		if (!tree -> ItemHasChildren(child_id))
		{					
   		    dynamic_cast <DirTreeCtrl *> (tree) ->add_branch(child_id, path + '/' +  filename);
		}				
			
       	cont = current_dir.GetNext(&filename);
        child_id = tree -> GetNextChild(id, cookie);
    }
    wxEndBusyCursor();
}

/**********************************************************************
   Function: DirDialog::make_file_vector
   Receives: (wxString & path_and_filespec)
   Returns:  (void)
   Description:
              Creates a vector of file paths corresponding to ALL the
              files that match the specifications of the received string.
**********************************************************************/

void DirDialog::make_file_vector(const wxString & path_and_filespec)
{
    wxLogNull no_message();
	wxDir current;
	wxString path = path_and_filespec.BeforeFirst('\t');
	wxString filespec = path_and_filespec.AfterFirst('\t');
	int test = 0;

	if(current.Open(path))
    {
        wxString filename;
    	
        bool cont = current.GetFirst(&filename, filespec, wxDIR_FILES | wxDIR_HIDDEN);
        wxString full_path = path + "/" + filename;

        while ( cont )
        {			        	
            add_to_panel_creation_files(full_path);
            cont = current.GetNext(&filename);
            full_path = path + "/" + filename;
        }

    }

}
/*********************************** EVENT DRIVEN FUNCTIONS ******************************************/

/**********************************************************************
   Function: DirDialog::OnChoice
   Receives: (wxCommandEvent&)
   Returns:  (void)
   Description:
              Processes the file search and panel creation based
              on the newly selected filespec.
**********************************************************************/

void DirDialog::OnChoice(wxCommandEvent& event)
{
     wxLogNull no_message;
     clear_text_box();
     begin_panel_creation();
     wxTreeItemId id = tree->GetSelection();

     wxString path = get_full_path(id);
     make_file_vector(path + "\t" +  get_filespec());
     reverse_panel_creation_files();
     start_panel_timer();
}
/**********************************************************************
   Function: ImageDialog::OnEnter
   Receives: wxCommandEvent&
   Returns:  N/A (void)
   Description:
            Processes the Enter Key event for the textctrl.
**********************************************************************/

void DirDialog::OnEnter(wxCommandEvent & event)
{
     wxLogNull no_message;
     begin_panel_creation();
     wxString path, filespec;

     if(text_box_value() != "")
     {
           parse_textctrl(path, filespec);
           make_file_vector(path + "\t" + filespec);
           reverse_panel_creation_files();
           start_panel_timer();
     }

}

/**********************************************************************
   Function: DirDialog::OnItemSelected
   Receives: (wxTreeEvent&)
   Returns:  (void)
   Description:
              Processes the file search and panel creation based
              on the newly selected filespec.
**********************************************************************/
void DirDialog::OnItemSelected(wxTreeEvent & event)
{
     wxLogNull no_message;
     clear_text_box();
     begin_panel_creation();
     wxString path = get_full_path(event.GetItem());
     make_file_vector(path + "\t" +  get_filespec());
     reverse_panel_creation_files();
     start_panel_timer();

}
