/************************************
Title : imagepanel.h
Authors : David Boys, Mike MacIntyre
Class Definition for  ImagePanel
Last updated : August 8, 2000
************************************/


#ifndef __IMAGEPANEL_H__
#define __IMAGEPANEL_H__

#include "wx/wxprec.h"
#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include "wx/panel.h"
#include "wx/image.h"
#include "wx/bitmap.h"
#include "wx/dc.h"
#include "wx/utils.h"

#include <map>
#include <vector>
using namespace std;

/*  Begin Members table

               Member                        |          Purpose

	wxImage               * image;           /   an image icon pointer to place on panel
	PreviewPanel          * preview;         /   a pointer pointing to PreviewPanel class
	vector <wxString>     & temp_selections; /   a temp vector to store the selections made
	const wxString          full_path;       /   the full_path of the image on the panel
	vector<ImagePanel *>  & panels;          /   a vector of panels that refers to itself
	bool                    activated;       /	 value for panel activation/highlighted
	wxTimer                 dbl_click_timer; /   a timer object to time click events
	
	clickableStaticBitmap * bm;              /	 a derived wxBitmap class to catch click events
	clickableStaticText   * file_name_label; /   a derived wxStaticText class to catch click events

    End  Members Table  */


	// timeout before starting to show the image (in case there's a double-click)
const int DCLICK_TIMEOUT = 200;

const int ID_timer = 1;


class clickableStaticBitmap;
class clickableStaticText;
class PreviewPanel;


class ImagePanel  : public wxPanel
{
public:
    ImagePanel(wxWindow * parent, const wxWindowID id, const wxPoint& pos,
	 		   const wxSize& size, long style , const wxString & path,
			   const int config_icon_size, vector <wxString> & temp,
			   vector<ImagePanel *> & panels, const wxImage * img,
			   PreviewPanel * preview);

	~ImagePanel();
	int label_width() const ;
	int label_height() const ;
	void load_preview_image();
	void clear_preview_image();
	void deselect_all();
	void select_panel();
	void reset();

	void OnLeftClick (wxMouseEvent & event);
	void OnLeftDClick (wxMouseEvent & event);
	void OnDClickTimeout (wxTimerEvent & event);
	 			
private:
	
	wxImage * image;
	PreviewPanel * preview;
	clickableStaticBitmap * bm;
	clickableStaticText * file_name_label;
	vector <wxString> & temp_selections;
	const wxString  full_path;
	vector<ImagePanel *> & panels;
	bool activated;
	wxTimer dbl_click_timer;
	
	
	DECLARE_EVENT_TABLE()
};

/// *** ClickableStaticBitmap Class ***

class clickableStaticBitmap : public wxStaticBitmap
{
public:
    clickableStaticBitmap (ImagePanel * parent, const wxWindowID id, const wxBitmap & bmp, const wxPoint & pos = wxDefaultPosition, const wxSize & size = wxDefaultSize);
    void OnClick (wxMouseEvent &);
    void OnDblClick (wxMouseEvent &);
};

/// *** ClickableStaticText Class ***

class clickableStaticText : public wxStaticText
{
public:
    clickableStaticText (ImagePanel * parent, const wxWindowID id, const wxString & txt, const wxPoint & pos = wxDefaultPosition, const wxSize & size = wxDefaultSize);
    void OnClick (wxMouseEvent &);
    void OnDblClick (wxMouseEvent &);
};

#endif
