///////////////////////////////////////////////////////////////////////////////////////////
// Authors: Eva Bunodiere, Yasmin Kossak, Mark Cecere
// mainFrame.cpp
// MainFrame Class functions definitions
// JPC36 Wx-View
// July 25th 2000
// Technical Advisor : Carlos Moreno
///////////////////////////////////////////////////////////////////////////////////////////

#include "mainFrame.h"

#include "ContentsFrame.h"
#include "AboutDialog.h"
#include "ZoomSettingsDialog.h"
#include "canvas.h"
#include "dirdialog.h"
#include "SetIntervalDialog.h"
#include "wx-view.h"
#include "languageDialog.h"
#include "ShowFilesDialog.h"

#include "filecanvas.h"
#include "imagepanel.h"
#include "imagedialog.h"
#include "category_treectrl.h"
#include "catdialog.h"
#include "AddFileToCategoryDlg.h"

#include "wx/utils.h"

//extern const Translator translation;
//extern User_preferences user_preferences;

BEGIN_EVENT_TABLE(MainFrame, wxFrame)
    EVT_MENU(ID_main_frame_properties, MainFrame::on_properties)
//    EVT_MENU(ID_main_frame_print, MainFrame::on_print)
    EVT_MENU(ID_main_frame_select_files_category, MainFrame::on_select_files_category)
	EVT_MENU(ID_main_frame_select_files_filename, MainFrame::on_select_files_filename)
    EVT_MENU(ID_main_frame_exit, MainFrame::on_exit)
    EVT_MENU(ID_main_frame_set_interval, MainFrame::on_set_interval)
    EVT_MENU(ID_main_frame_show_files, MainFrame::on_show_files)
	EVT_MENU(ID_main_frame_run, MainFrame::on_run)
    EVT_MENU(ID_main_frame_rotate, MainFrame::on_rotate)
    EVT_MENU(ID_main_frame_horizontal_flip, MainFrame::on_horizontal_flip)
    EVT_MENU(ID_main_frame_vertical_flip, MainFrame::on_vertical_flip)
    EVT_MENU(ID_main_frame_zoom_in, MainFrame::on_zoom_in)
    EVT_MENU(ID_main_frame_zoom_out, MainFrame::on_zoom_out)
    EVT_MENU(ID_main_frame_increase_contrast, MainFrame::on_increase_contrast)
    EVT_MENU(ID_main_frame_decrease_contrast, MainFrame::on_decrease_contrast)
    EVT_MENU(ID_main_frame_increase_brightness, MainFrame::on_increase_brightness)
    EVT_MENU(ID_main_frame_decrease_brightness, MainFrame::on_decrease_brightness)
    EVT_MENU(ID_main_frame_original_image, MainFrame::on_original_image)
	EVT_MENU(ID_main_frame_language, MainFrame::on_language)
    EVT_MENU(ID_toggle_toolbar, MainFrame::on_toggle_toolbar)
//    EVT_MENU(ID_main_frame_directory, MainFrame::on_directory)
	EVT_MENU(ID_main_frame_zoom_settings, MainFrame::on_zoom_settings)
    EVT_MENU(ID_main_frame_contents, MainFrame::on_contents)
    EVT_MENU(ID_main_frame_about, MainFrame::on_about)

    EVT_MENU (ID_20pixels, MainFrame::on_20pixels)
    EVT_MENU (ID_24pixels, MainFrame::on_24pixels)
    EVT_MENU (ID_28pixels, MainFrame::on_28pixels)
    EVT_MENU (ID_32pixels, MainFrame::on_32pixels)
    EVT_MENU (ID_40pixels, MainFrame::on_40pixels)
    EVT_MENU (ID_48pixels, MainFrame::on_48pixels)
    EVT_MENU (ID_56pixels, MainFrame::on_56pixels)
    EVT_MENU (ID_64pixels, MainFrame::on_64pixels)
    EVT_MENU (ID_80pixels, MainFrame::on_80pixels)
    EVT_MENU (ID_100pixels, MainFrame::on_100pixels)
END_EVENT_TABLE()


/******************************************************************************************
	Function: MainFrame::MainFrame
	Receives: N/A (constructor)
	Returns: N/A (constructor)
	Description: 
		Calls the wxFrame constructor to create a new main frame for the application. Creates
		new wxMenus to attach to the frame as a wxMenuBar. Calls init_toolbar to create the 
		toolbar. Initializes the canvas member variable to a new Canvas object.
*******************************************************************************************/
MainFrame::MainFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
#ifdef __WXMSW__
    : wxFrame (NULL, -1, title, pos, size, wxMAXIMIZE)  // this style only supported on wxMSW
#else
    : wxFrame (NULL, -1, title, pos, size)
#endif
{
	wxMenu * menu_file =         new wxMenu;
	wxMenu * menu_select_files = new wxMenu;
	wxMenu * menu_slide_show =   new wxMenu;
	wxMenu * menu_view =         new wxMenu;
	wxMenu * menu_help =         new wxMenu;
	wxMenu * menu_options =		 new wxMenu;
	wxMenu * menu_icon_sizes =   new wxMenu;

	menu_select_files->Append(ID_main_frame_select_files_filename, translation["Select Files by &Filename...\tCtrl-F"] );
	menu_select_files->Append(ID_main_frame_select_files_category, translation["&Select Files by &Category...\tCtrl-C"] );

	menu_file->Append(ID_main_frame_properties, translation["Specify Category...\tF2"]);
//	menu_file->Append(ID_main_frame_print, translation["&Print   \tCtrl-P"] );
//      maybe in version 1.1?

	menu_file->Append(ID_main_frame_select_files, translation["&Select Files"],
						menu_select_files);
    menu_file->AppendSeparator();
    menu_file->Append(ID_main_frame_exit, translation["E&xit   \tAlt-X"] );

	
	menu_slide_show->Append(ID_main_frame_set_interval, translation["&Set Interval...\tCtrl-I"] ); 	
	menu_slide_show->Append(ID_main_frame_show_files, translation["Show &Files...\tCtrl-S"]);
	menu_slide_show->AppendSeparator();
	menu_slide_show->Append(ID_main_frame_run, translation["&Run   \tCtrl-U"] );
	
	
	menu_view->Append(ID_main_frame_rotate, translation["&Rotate...\tCtrl-O"] );
	menu_view->AppendSeparator();
	menu_view->Append(ID_main_frame_horizontal_flip, translation["&Horizontal Flip   \tCtrl-H"] );
	menu_view->Append(ID_main_frame_vertical_flip, translation["&Vertical Flip   \tCtrl-V"] );
    menu_view->AppendSeparator();
	menu_view->Append(ID_main_frame_zoom_in, translation["Zoom &In\tPgUp"] );
	menu_view->Append(ID_main_frame_zoom_out, translation["Zoom &Out\tPgDn"] );
    menu_view->AppendSeparator();
    menu_view->Append(ID_main_frame_increase_contrast, translation["I&ncrease Contrast   \tCtrl-R"] );
	menu_view->Append(ID_main_frame_decrease_contrast, translation["&Decrease Contrast   \tAlt-R"] );
    menu_view->AppendSeparator();
    menu_view->Append(ID_main_frame_increase_brightness, translation["In&crease Brightness   \tCtrl-B"] );
    menu_view->Append(ID_main_frame_decrease_brightness, translation["Decrea&se Brightness   \tAlt-B"] );
    menu_view->AppendSeparator();
	menu_view->Append(ID_main_frame_original_image, translation["Ori&ginal Image   \tCtrl-G"] );
	

	menu_icon_sizes->Append (ID_20pixels, "20", "", true);
	menu_icon_sizes->Append (ID_24pixels, "24", "", true);
	menu_icon_sizes->Append (ID_28pixels, "28", "", true);
	menu_icon_sizes->Append (ID_32pixels, "32", "", true);
	menu_icon_sizes->Append (ID_40pixels, "40", "", true);
	menu_icon_sizes->Append (ID_48pixels, "48", "", true);
	menu_icon_sizes->Append (ID_56pixels, "56", "", true);
	menu_icon_sizes->Append (ID_64pixels, "64", "", true);
	menu_icon_sizes->Append (ID_80pixels, "80", "", true);
	menu_icon_sizes->Append (ID_100pixels, "100", "", true);

	menu_options->Append(ID_main_frame_language, translation["&Language..."] );
    menu_options->AppendSeparator();
	menu_options->Append(ID_toggle_toolbar, translation["View &Toolbar"], "", true );
	menu_options->Check(ID_toggle_toolbar, user_preferences.view_toolbar);
							
    menu_options->AppendSeparator();
	menu_options->Append(ID_main_frame_zoom_settings, translation["&Zoom Settings...\tCtrl-Z"] );
	
	menu_options->AppendSeparator();
	menu_options->Append (ID_options_icon_size, translation["Iconized images size (pixels)"], menu_icon_sizes);

//	menu_options->AppendSeparator();
//	menu_options->Append(ID_main_frame_directory, translation["&Directory..."] );

	menu_help->Append(ID_main_frame_contents, translation["&Contents...\tF1"] );
    menu_help->AppendSeparator();
	menu_help->Append(ID_main_frame_about, translation["&About..."]);

	
	// Disable all image dependent menu items: will be enabled when a pic is loaded.

	menu_file->Enable(ID_main_frame_properties, false);
//	menu_file->Enable(ID_main_frame_print, false);
	menu_slide_show->Enable(ID_main_frame_set_interval, false);
	menu_slide_show->Enable(ID_main_frame_show_files, false);
	menu_slide_show->Enable(ID_main_frame_run, false);
	menu_view->Enable(ID_main_frame_rotate, false);
	menu_view->Enable(ID_main_frame_horizontal_flip, false);
	menu_view->Enable(ID_main_frame_vertical_flip, false);
	menu_view->Enable(ID_main_frame_zoom_in, false);
	menu_view->Enable(ID_main_frame_zoom_out, false);
	menu_view->Enable(ID_main_frame_increase_contrast, false);
	menu_view->Enable(ID_main_frame_decrease_contrast, false);
	menu_view->Enable(ID_main_frame_increase_brightness, false);
	menu_view->Enable(ID_main_frame_decrease_brightness, false);
	menu_view->Enable(ID_main_frame_original_image, false);

    // now append the freshly created menu to the menu bar...
    
	wxMenuBar *menu_bar = new wxMenuBar();

	menu_bar->Append(menu_file, translation["&File"] );
	menu_bar->Append(menu_slide_show, translation["&Slide Show"] );
	menu_bar->Append(menu_view, translation["&View"] );
	menu_bar->Append(menu_options, translation["&Options"]);
    menu_bar->Append(menu_help, translation["&Help"]);

    // ... and attach this menu bar to the frame
    SetMenuBar( menu_bar );

	// Creating the toolbar
	
	init_toolbar();	
	init_popup_menu();

	// Activate available handlers for image file formats
	wxImage::AddHandler( new wxPNGHandler );
	wxImage::AddHandler( new wxJPEGHandler );
	wxImage::AddHandler( new wxTIFFHandler );
	wxImage::AddHandler( new wxGIFHandler );
  	wxImage::AddHandler( new wxPCXHandler );
  	wxImage::AddHandler( new wxPNMHandler );

	// Create a new canvas to load pics onto
	canvas = new Canvas(this, ID_canvas, wxSize(GetSize()));
	canvas->change_zoom_type(user_preferences.zoom_type);
	slide_show = new Slide_show(canvas);
	image_open = false;
	
	toolbar->Show (user_preferences.view_toolbar);
	
	switch (user_preferences.icon_size)
	{
	    case 20:
	        menu_bar->Check (ID_20pixels, true);
	        break;

	    case 24:
	        menu_bar->Check (ID_24pixels, true);
	        break;

	    case 28:
	        menu_bar->Check (ID_28pixels, true);
	        break;

	    case 32:
	        menu_bar->Check (ID_32pixels, true);
	        break;

	    case 40:
	        menu_bar->Check (ID_40pixels, true);
	        break;

	    case 48:
	        menu_bar->Check (ID_48pixels, true);
	        break;

	    case 56:
	        menu_bar->Check (ID_56pixels, true);
	        break;

	    case 64:
	        menu_bar->Check (ID_64pixels, true);
	        break;

	    case 80:
	        menu_bar->Check (ID_80pixels, true);
	        break;

	    case 100:
	        menu_bar->Check (ID_100pixels, true);
	        break;

	    default:
	        menu_bar->Check (ID_32pixels, true);
	        break;
	}
}

/******************************************************************************************
	Function: MainFrame::init_toolbar
	Receives: N/A
	Returns: bool
	Description: 
		Creates a new wxToolbar. Initializes wxBitmaps for the toolbar buttons. Disables all
		image-dependent toolbar buttons. Deletes the toolbar bitmaps.
*******************************************************************************************/
bool MainFrame::init_toolbar()
{
	
	toolbar = GetToolBar();
    delete toolbar;

    SetToolBar(NULL);

    long style = wxNO_BORDER | wxTB_HORIZONTAL | wxTB_DOCKABLE ;

    toolbar = CreateToolBar(style, ID_toolbar);
    toolbar->SetMargins( 4, 4 );

    // Set up toolbar

	wxBitmap* toolbar_bitmaps[14];
	toolbar_bitmaps[0] = new wxBitmap("open.bmp", wxBITMAP_TYPE_BMP);

	// Initialize toolbar bitmaps

	toolbar_bitmaps[1] = new wxBitmap( "rescan.bmp", wxBITMAP_TYPE_BMP );
//    toolbar_bitmaps[2] = new wxBitmap( "print.bmp", wxBITMAP_TYPE_BMP );
	toolbar_bitmaps[2] = new wxBitmap( "rotate.bmp", wxBITMAP_TYPE_BMP);
	toolbar_bitmaps[3] = new wxBitmap( "horizontalFlip.bmp", wxBITMAP_TYPE_BMP );
	toolbar_bitmaps[4] = new wxBitmap( "verticalFlip.bmp", wxBITMAP_TYPE_BMP );
	toolbar_bitmaps[5] = new wxBitmap( "magnify.bmp", wxBITMAP_TYPE_BMP );
	toolbar_bitmaps[6] = new wxBitmap( "reduce.bmp", wxBITMAP_TYPE_BMP );
    toolbar_bitmaps[7] = new wxBitmap( "add_contrast.bmp", wxBITMAP_TYPE_BMP );
    toolbar_bitmaps[8] = new wxBitmap( "remove_contrast.bmp", wxBITMAP_TYPE_BMP );
	toolbar_bitmaps[9] = new wxBitmap( "add_brightness.bmp", wxBITMAP_TYPE_BMP );
    toolbar_bitmaps[10] = new wxBitmap( "remove_brightness.bmp", wxBITMAP_TYPE_BMP );
	toolbar_bitmaps[11] = new wxBitmap( "camera.bmp", wxBITMAP_TYPE_BMP );
	toolbar_bitmaps[12] = new wxBitmap( "help.bmp", wxBITMAP_TYPE_BMP);
	
	toolbar_bitmaps[13] = new wxBitmap ("categories.bmp", wxBITMAP_TYPE_BMP);
	
	int width = 16;

	int currentX = 5;

	//Add toolbar buttons

	toolbar->AddTool(ID_main_frame_select_files_filename, *(toolbar_bitmaps[0]), wxNullBitmap, FALSE, currentX, -1,
					(wxObject *) NULL, translation["Select Files by filename"] );

	currentX += width + 5;
	toolbar->AddTool(ID_main_frame_select_files_category, *(toolbar_bitmaps[13]), wxNullBitmap, FALSE, currentX, -1,
					(wxObject *) NULL, translation["Select Files by category"] );

	currentX += width + 5;
	toolbar->AddTool(ID_main_frame_original_image, *(toolbar_bitmaps[1]), wxNullBitmap, FALSE, currentX, -1,
					(wxObject *) NULL, translation["Original Image"] );

//	currentX += width + 5;
//	toolbar->AddTool(ID_main_frame_print, *(toolbar_bitmaps[2]), wxNullBitmap, FALSE, currentX, -1,
//					(wxObject *) NULL, translation["Print"] );

	toolbar->AddSeparator();
	currentX += width + 5;
	toolbar->AddTool(ID_main_frame_rotate, *(toolbar_bitmaps[2]), wxNullBitmap, FALSE, currentX, -1,
					(wxObject *) NULL, translation["Rotate Image"] );

	currentX += width + 5;

	toolbar->AddTool(ID_main_frame_horizontal_flip, *(toolbar_bitmaps[3]), wxNullBitmap, FALSE, currentX, -1,
					(wxObject *) NULL, translation["Horizontal Flip"] );

	currentX += width + 5;
	toolbar->AddTool(ID_main_frame_vertical_flip, *(toolbar_bitmaps[4]), wxNullBitmap, FALSE, currentX, -1,
					(wxObject *) NULL, translation["Vertical Flip"] );

	currentX += width + 5;
	toolbar->AddTool(ID_main_frame_zoom_in, *(toolbar_bitmaps[5]), wxNullBitmap, FALSE, currentX, -1,
					(wxObject *) NULL, translation["Zoom In"] );

	currentX += width + 5;
	toolbar->AddTool(ID_main_frame_zoom_out, *(toolbar_bitmaps[6]), wxNullBitmap, FALSE, currentX, -1,
					(wxObject *) NULL, translation["Zoom Out"] );

	currentX += width + 5;
	toolbar->AddTool(ID_main_frame_increase_contrast, *(toolbar_bitmaps[7]), wxNullBitmap, FALSE, currentX, -1,
					(wxObject *) NULL, translation["Increase Contrast"] );
					
	currentX += width + 5;
	toolbar->AddTool(ID_main_frame_decrease_contrast, *(toolbar_bitmaps[8]), wxNullBitmap, FALSE, currentX, -1,
					(wxObject *) NULL, translation["Decrease Contrast"] );
	
	currentX += width + 5;
	toolbar->AddTool(ID_main_frame_increase_brightness, *(toolbar_bitmaps[9]), wxNullBitmap, FALSE, currentX, -1,
					(wxObject *) NULL, translation["Increase Brightness"] );
	
	currentX += width + 5;
	toolbar->AddTool(ID_main_frame_decrease_brightness, *(toolbar_bitmaps[10]), wxNullBitmap, FALSE, currentX, -1,
					(wxObject *) NULL, translation["Decrease Brightness"] );
	
	currentX += width + 5;
	
	toolbar->AddSeparator();
	
	toolbar->AddTool(ID_main_frame_run, *(toolbar_bitmaps[11]), wxNullBitmap, FALSE, currentX, -1,
					(wxObject *) NULL, translation["Run Slide Show"] );

	currentX += width + 5;
	toolbar->AddTool(ID_main_frame_contents, *(toolbar_bitmaps[12]), wxNullBitmap, FALSE, currentX, -1,
			(wxObject *) NULL, translation["Help"] );

	toolbar->EnableTool(ID_main_frame_original_image, false);
//	toolbar->EnableTool(ID_main_frame_print, false);
	toolbar->EnableTool(ID_main_frame_rotate, false);
	toolbar->EnableTool(ID_main_frame_horizontal_flip, false);
	toolbar->EnableTool(ID_main_frame_vertical_flip, false);
	toolbar->EnableTool(ID_main_frame_vertical_flip, false);
	toolbar->EnableTool(ID_main_frame_zoom_in, false);
	toolbar->EnableTool(ID_main_frame_zoom_out, false);
	toolbar->EnableTool(ID_main_frame_increase_contrast, false);
	toolbar->EnableTool(ID_main_frame_decrease_contrast, false);
	toolbar->EnableTool(ID_main_frame_increase_brightness, false);
	toolbar->EnableTool(ID_main_frame_decrease_brightness, false);
	toolbar->EnableTool(ID_main_frame_run, false);

	// Cause the changes to take effect

	toolbar->Realize();

	// Can delete the bitmaps since they're reference counted

	for (int i = 0; i < 8; i++)
	  delete toolbar_bitmaps[i];

	return true;
}


/******************************************************************************************
	Function: MainFrame::init_popup_menu
	Receives: N/A
	Returns: bool
	Description:
		Creates and initializes the image popup menu
*******************************************************************************************/
void MainFrame::init_popup_menu ()
{
    popup_menu = new wxMenu();

    popup_menu->Append(ID_main_frame_properties, (wxString(translation["Specify Category...\tF2"]).BeforeFirst('\t')).c_str());
	
    popup_menu->AppendSeparator();


	popup_menu->Append(ID_main_frame_rotate, wxString(translation["&Rotate...\tCtrl-O"]).BeforeFirst('\t') );
	popup_menu->AppendSeparator();
	popup_menu->Append(ID_main_frame_horizontal_flip, wxString(translation["&Horizontal Flip   \tCtrl-H"]).BeforeFirst('\t') );
	popup_menu->Append(ID_main_frame_vertical_flip, wxString(translation["&Vertical Flip   \tCtrl-V"]).BeforeFirst('\t')  );
    popup_menu->AppendSeparator();
	popup_menu->Append(ID_main_frame_zoom_in, wxString(translation["Zoom &In\tPgUp"]).BeforeFirst('\t') );
	popup_menu->Append(ID_main_frame_zoom_out, wxString(translation["Zoom &Out\tPgDn"]).BeforeFirst('\t') );
    popup_menu->AppendSeparator();
    popup_menu->Append(ID_main_frame_increase_contrast, wxString(translation["I&ncrease Contrast   \tCtrl-R"]).BeforeFirst('\t') );
	popup_menu->Append(ID_main_frame_decrease_contrast, wxString(translation["&Decrease Contrast   \tAlt-R"]).BeforeFirst('\t') );
    popup_menu->AppendSeparator();
    popup_menu->Append(ID_main_frame_increase_brightness, wxString(translation["In&crease Brightness   \tCtrl-B"]).BeforeFirst('\t') );
    popup_menu->Append(ID_main_frame_decrease_brightness, wxString(translation["Decrea&se Brightness   \tAlt-B"]).BeforeFirst('\t') );
    popup_menu->AppendSeparator();
	popup_menu->Append(ID_main_frame_original_image, wxString(translation["Ori&ginal Image   \tCtrl-G"]).BeforeFirst('\t') );
	
    popup_menu->AppendSeparator();
	popup_menu->Append(ID_main_frame_zoom_settings, wxString(translation["&Zoom Settings...\tCtrl-Z"]).BeforeFirst('\t') );
}

/******************************************************************************************
	Function: MainFrame::on_properties
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		
*******************************************************************************************/
void MainFrame::on_properties(wxCommandEvent& )
{
	AddFileToCategoryDlg * dlg = new AddFileToCategoryDlg (this, translation["Add file to category"], image_listing[file_index]);
	dlg->ShowModal();

    dlg->Destroy();
}

/******************************************************************************************
	Function: MainFrame::on_print
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		
*******************************************************************************************/
/*void MainFrame::on_print(wxCommandEvent& )
{}
*/

/******************************************************************************************
	Function: MainFrame::on_select_files_category
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		
*******************************************************************************************/
void MainFrame::on_select_files_category(wxCommandEvent& )
{
    const int scr_x = wxSystemSettings::GetSystemMetric(wxSYS_SCREEN_X);
	const int scr_y = wxSystemSettings::GetSystemMetric(wxSYS_SCREEN_Y);
	
	wxRect get_dialog_size_and_position (int, int);
	const wxRect size_and_pos = get_dialog_size_and_position (scr_x, scr_y);
	
	CatDialog * dialog = new CatDialog (this, -1, translation["Open"],
	                                    wxPoint (size_and_pos.GetLeft(), size_and_pos.GetTop()),
	                                    wxSize (size_and_pos.GetWidth(), size_and_pos.GetHeight()),
	                                    user_preferences.filename.c_str(),
	                                    user_preferences.fullpath.c_str(),
	                                    user_preferences.icon_size);

	if (dialog->ShowModal() == wxID_OK)
	{
	    image_listing = dialog->get_image_files();
	
		if (!image_listing.empty())
		{
    		file_index = 0;
    		canvas->load_file(image_listing[file_index]);
    		// SetTitle (wxString(translation["Wx-View 1.0"]) + " - [" + image_listing[file_index] + "]");
    		// Canvas object does it
    		
			// Enable the menu items and toolbar
    		image_open = true;
			enable_image_items();
		}
	}

	dialog->Destroy();
}


/******************************************************************************************
	Function: MainFrame::on_select_files_filename
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		Creates a new DirDialog for the user to select files.  When the dialog returns wxID_OK,
		get_image_files returns a vector of strings. The first selection is then loaded onto the
		canvas.
*******************************************************************************************/
void MainFrame::on_select_files_filename(wxCommandEvent& )
{	
	const int scr_x = wxSystemSettings::GetSystemMetric(wxSYS_SCREEN_X);
	const int scr_y = wxSystemSettings::GetSystemMetric(wxSYS_SCREEN_Y);
	
	wxRect get_dialog_size_and_position (int, int);
	const wxRect size_and_pos = get_dialog_size_and_position (scr_x, scr_y);

	DirDialog * dialog = new DirDialog (this, -1, translation["Open"],
	                                    wxPoint (size_and_pos.GetLeft(), size_and_pos.GetTop()),
	                                    wxSize (size_and_pos.GetWidth(), size_and_pos.GetHeight()),
                                        user_preferences.filename.c_str(),
                                        user_preferences.fullpath.c_str(),
                                        user_preferences.icon_size);

	if (dialog->ShowModal() == wxID_OK)
	{
	    image_listing = dialog->get_image_files();
	
		if (!image_listing.empty())
		{
    		file_index = 0;
    		canvas->load_file(image_listing[file_index]);
    		// SetTitle (wxString(translation["Wx-View 1.0"]) + " - [" + image_listing[file_index] + "]");
    		// Canvas object does it
    		
			// Enable the menu items and toolbar
    		image_open = true;
			enable_image_items();
		}
	}

	dialog->Destroy();
}


/******************************************************************************************
	Function: MainFrame::on_exit
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		Ends the application
*******************************************************************************************/
void MainFrame::on_exit(wxCommandEvent& )
{
    Close(true);
}


/******************************************************************************************
	Function: MainFrame::on_set_interval
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		
*******************************************************************************************/
void MainFrame::on_set_interval(wxCommandEvent& )
{
    SetIntervalDialog * dialog = new SetIntervalDialog (this, ID_main_frame_set_interval,
            translation["Set Interval"], user_preferences.interval);
	
	if (dialog->ShowModal() == wxID_OK)
	{
	    user_preferences.interval = dialog->read_interval();
	    write_config_file(translation.return_language());
	}
	
	dialog->Destroy();
}

/******************************************************************************************
	Function: MainFrame::on_show_files
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		
*******************************************************************************************/
void MainFrame::on_show_files(wxCommandEvent& )
{
    ShowFilesDialog * files_dialog = new ShowFilesDialog(this, -1, translation["List of Selected Files"], image_listing);

    files_dialog->ShowModal();

    files_dialog->Destroy();
}

/******************************************************************************************
	Function: MainFrame::on_run
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		
*******************************************************************************************/
void MainFrame::on_run(wxCommandEvent& )
{
    slide_show-> start_slide_show(user_preferences.interval, image_listing);
    //file_index = 0;
    //canvas->load_file(image_listing[file_index]);

}
/******************************************************************************************
	Function: MainFrame::on_rotate
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		Calls the Canvas function OnRotate.
*******************************************************************************************/
void MainFrame::on_rotate(wxCommandEvent& )
{
	canvas->on_rotate();
}


/******************************************************************************************
	Function: MainFrame::on_horizontal_flip
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		Calls the Canvas function OnHFlip.
*******************************************************************************************/
void MainFrame::on_horizontal_flip(wxCommandEvent& )
{
	canvas->on_h_flip();
}


/******************************************************************************************
	Function: MainFrame::on_vertical_flip
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		Calls the Canvas function OnVFlip
*******************************************************************************************/
void MainFrame::on_vertical_flip(wxCommandEvent& )
{
	canvas->on_v_flip();
}


/******************************************************************************************
	Function: MainFrame::on_zoom_in
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		Calls the Canvas function ZoomCenter.
*******************************************************************************************/
void MainFrame::on_zoom_in(wxCommandEvent& )
{
    canvas->zoom_center();
}


/******************************************************************************************
	Function: MainFrame::on_zoom_out
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		Calls the Canvas function ZoomOut
*******************************************************************************************/
void MainFrame::on_zoom_out(wxCommandEvent& )
{
	canvas->zoom_out();			
}


/******************************************************************************************
	Function: MainFrame::on_original_image
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		Calls the Canvas function Original.
*******************************************************************************************/
void MainFrame::on_original_image( wxCommandEvent& )
{
	canvas->original();
}


/******************************************************************************************
	Function: MainFrame::on_zoom_settings
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		Creates a new ZoomSettingsDialog. When the dialog return wxID_OK, the zoom type selected
		is returned and changed, as well as the zoom factor.
*******************************************************************************************/
void MainFrame::on_zoom_settings(wxCommandEvent& )
{
    ZoomSettingsDialog * settings = new ZoomSettingsDialog (this, -1, translation["Zoom Settings"], user_preferences.zoom_factor, user_preferences.zoom_type);
	
	if (settings->ShowModal() == wxID_OK)
	{
	    user_preferences.zoom_type = settings->zoom_by_interpolation();
	    user_preferences.zoom_factor = settings->zoom_factor();
	    write_config_file(translation.return_language());
	    canvas->change_zoom_type(user_preferences.zoom_type);
	    canvas->change_zoom_factor(user_preferences.zoom_factor);
	}
	
	settings->Destroy();
}


/******************************************************************************************
	Function: MainFrame::on_language
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		Creates a new Language_Dialog. When the dialog return wxID_OK, the language selected is 
		returned and written to file.
*******************************************************************************************/
void MainFrame::on_language(wxCommandEvent& WXUNUSED(event))
{
    Language_Dialog * settings = new Language_Dialog (this, -1, translation["Select a Language"]);	

	if (settings->ShowModal() == wxID_OK)
	{
	    const char* lang = settings->language_selected();
	    write_config_file(lang);
	}
	
	settings->Destroy();
}

/******************************************************************************************
	Function: MainFrame::on_toggle_toolbar
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		Shows the toolbar, passing the status of the checkable menu item, true or false.
*******************************************************************************************/
void MainFrame::on_toggle_toolbar(wxCommandEvent &)
{
	user_preferences.view_toolbar = GetMenuBar()->IsChecked(ID_toggle_toolbar);
	toolbar->Show (user_preferences.view_toolbar);
    write_config_file(translation.return_language());
}


/******************************************************************************************
	Function: MainFrame::on_directory
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		
*******************************************************************************************/
/*
void MainFrame::on_directory(wxCommandEvent& WXUNUSED(event))
{}

*/
/******************************************************************************************
	Function: MainFrame::on_contents
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		Creates a new ContentsFrame object. Shows it and calls ShowHelp passing the translation
		of the document in the current langauge.
*******************************************************************************************/
void MainFrame::on_contents(wxCommandEvent& WXUNUSED(event))
{
    ContentsFrame * contents = new ContentsFrame (this, ID_contents_frame, translation["Help"],
        wxDefaultPosition, wxSize(520,300), wxDEFAULT_FRAME_STYLE);
    contents->Show(true);
    contents->ShowHelp(translation["Help File"]);
}


/******************************************************************************************
	Function: MainFrame::on_about
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		Creates a new AboutDialog.
*******************************************************************************************/
void MainFrame::on_about(wxCommandEvent& WXUNUSED(event))
{	
	AboutDialog * about = new AboutDialog (this, ID_main_frame_about, translation["About Wx-View 1.0"], translation["About Text"], translation["OK"]);
	about->ShowModal();

	about->Destroy();
}


/******************************************************************************************
	Function: MainFrame::write_config_file
	Receives: N/A
	Returns: N/A
	Description:
		Reads the configuration file. Reads the file, adding those values to the settings structure.
*******************************************************************************************/
void MainFrame::write_config_file(const string & lang) const
{
    user_preferences.write_to_file(lang);
}


/******************************************************************************************
	Function: MainFrame::on_increase_contrast
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		
*******************************************************************************************/
void MainFrame::on_increase_contrast(wxCommandEvent& )
{
    canvas->on_plus_contrast();
}

/******************************************************************************************
	Function: MainFrame::on_decrease_contrast
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		
*******************************************************************************************/
void MainFrame::on_decrease_contrast(wxCommandEvent& )
{
    canvas->on_minus_contrast();
}

/******************************************************************************************
	Function: MainFrame::on_increase_brightness
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		
*******************************************************************************************/
void MainFrame::on_increase_brightness(wxCommandEvent& )
{
    canvas->on_bright();
}

/******************************************************************************************
	Function: MainFrame::on_decrease_brightness
	Receives: wxCommandEvent &
	Returns: N/A
	Description: 
		
*******************************************************************************************/
void MainFrame::on_decrease_brightness(wxCommandEvent& )
{
    canvas->on_dim();
}

/******************************************************************************************
	Function: MainFrame::enable_image_items
	Receives: N/A
	Returns: N/A
	Description: 
		Gets the wxMenuBar and wxToolBar for the frame, enabling all image-dependant items.
*******************************************************************************************/
void MainFrame::enable_image_items()
{
	wxMenuBar *menu_bar = GetMenuBar();

	wxMenu * menu_file =         menu_bar->GetMenu(0);
	wxMenu * menu_slide_show =   menu_bar->GetMenu(1);
	wxMenu * menu_view =         menu_bar->GetMenu(2);

	menu_file->Enable(ID_main_frame_properties, true);
//	menu_file->Enable(ID_main_frame_print, true);

	menu_slide_show->Enable(ID_main_frame_set_interval, true);
	menu_slide_show->Enable(ID_main_frame_show_files, true);
	menu_slide_show->Enable(ID_main_frame_run, true);
	
	menu_view->Enable(ID_main_frame_rotate, true);
	menu_view->Enable(ID_main_frame_horizontal_flip, true);
	menu_view->Enable(ID_main_frame_vertical_flip, true);
	menu_view->Enable(ID_main_frame_zoom_in, true);
	menu_view->Enable(ID_main_frame_zoom_out, true);
	menu_view->Enable(ID_main_frame_increase_contrast, true);
	menu_view->Enable(ID_main_frame_decrease_contrast, true);
	menu_view->Enable(ID_main_frame_increase_brightness, true);
	menu_view->Enable(ID_main_frame_decrease_brightness, true);
	menu_view->Enable(ID_main_frame_original_image, true);


	wxToolBar * toolbar = GetToolBar();

	toolbar->EnableTool(ID_main_frame_original_image, true);
//	toolbar->EnableTool(ID_main_frame_print, true);
	toolbar->EnableTool(ID_main_frame_rotate, true);
	toolbar->EnableTool(ID_main_frame_horizontal_flip, true);
	toolbar->EnableTool(ID_main_frame_vertical_flip, true);
	toolbar->EnableTool(ID_main_frame_vertical_flip, true);
	toolbar->EnableTool(ID_main_frame_zoom_in, true);
	toolbar->EnableTool(ID_main_frame_zoom_out, true);
	toolbar->EnableTool(ID_main_frame_increase_contrast, true);
	toolbar->EnableTool(ID_main_frame_decrease_contrast, true);
	toolbar->EnableTool(ID_main_frame_increase_brightness, true);
	toolbar->EnableTool(ID_main_frame_decrease_brightness, true);
	toolbar->EnableTool(ID_main_frame_run, true);

	toolbar->Realize();
}


/******************************************************************************************
	Function: MainFrame::pop_up_menu
	Receives: wxMouseEvent &
	Returns: N/A
	Description: 
		Gets the wxPoint where the user right-clicked, creating and showing a pop-up menu at 
		that location.
*******************************************************************************************/
void MainFrame::pop_up_menu (wxMouseEvent& event)
{
    wxPoint pos = event.GetPosition();
    popup_menu->Enable(ID_main_frame_properties, image_open);
	popup_menu->Enable(ID_main_frame_rotate, image_open);
	popup_menu->Enable(ID_main_frame_horizontal_flip, image_open);
	popup_menu->Enable(ID_main_frame_vertical_flip, image_open);
	popup_menu->Enable(ID_main_frame_zoom_in, image_open);
	popup_menu->Enable(ID_main_frame_zoom_out, image_open);
	popup_menu->Enable(ID_main_frame_increase_contrast, image_open);
	popup_menu->Enable(ID_main_frame_decrease_contrast, image_open);
	popup_menu->Enable(ID_main_frame_increase_brightness, image_open);
	popup_menu->Enable(ID_main_frame_decrease_brightness, image_open);
	popup_menu->Enable(ID_main_frame_original_image, image_open);
	popup_menu->Enable(ID_main_frame_zoom_settings, image_open);

	PopupMenu (popup_menu, pos);
}


/******************************************************************************************
	Function: MainFrame::on_key_down
	Receives: wxKeyEvent &
	Returns: N/A
	Description:
		Receives a wxKeyEvent, determines which key was pressed, and calls the appropriate
		canvas functions.
*******************************************************************************************/
void MainFrame::on_key_down(wxKeyEvent & event)
{
    switch(event.KeyCode())
    {
        case WXK_SPACE:    //Space Bar
            if (! image_listing.empty())    // BUG FIX (C.M.) -- crashed if no images selected
            {
                if (file_index < image_listing.size() - 1)
                {
                    canvas->load_file(image_listing[++file_index]);
        	    	    // SetTitle (wxString(translation["Wx-View 1.0"]) + " - [" + image_listing[file_index] + "]");
            		    // Canvas object does it
                }
                else
                {
                    file_index = 0;
                    canvas->load_file(image_listing[file_index]);
                		//SetTitle (wxString(translation["Wx-View 1.0"]) + " - [" + image_listing[file_index] + "]");
        		        // Canvas object does it
                }
            }
            break;

        case WXK_ESCAPE:
            canvas->original();
            break;

        case '+':
        case WXK_ADD:   //Keypad Add
        case WXK_PRIOR: // PgUp
            canvas->zoom_center();
            break;

        case '-':
        case WXK_SUBTRACT:
        case WXK_NEXT:  // PgDn
            canvas->zoom_out();
            break;

        case WXK_LEFT:

            if (event.ControlDown())
            {
                canvas->shift_left();
            }
            else
            {
                canvas->shift_left(0.05);
            }
            break;

        case WXK_UP:
            if (event.ControlDown())
            {
                canvas->shift_up();
            }
            else
            {
                canvas->shift_up(0.05);
            }
            break;

        case WXK_RIGHT:
            if ( event.ControlDown())
            {
                canvas->shift_right();
            }
            else
            {
                canvas->shift_right(0.05);
            }
            break;

        case WXK_DOWN:
            if ( event.ControlDown())
            {
                canvas->shift_down();
            }
            else
            {
                canvas->shift_down(0.05);
            }
            break;

        default:            // Someone else should handle this event (shortcut keys, etc.)
            event.Skip();
    }
}


/******************************************************************************************
	Function: get_dialog_size_and_position
	Receives: scr_x, scr_y (screen size)
	Returns: wxRect representing the size and position of the dialog (file selection dialog)
	Description:
	    Finds the appropriate size and position for the file selection dialog, according
	    to the screen size (resolution) -- this might be changed in a near future, to
	    allow the user to select the size (storing it in the configuration file)
*******************************************************************************************/
wxRect get_dialog_size_and_position (int scr_x, int scr_y)
{
    switch (scr_x)
    {
        case 640:
            return wxRect (wxPoint (20,10), wxSize (600, 400));
            break;

        case 800:
            return wxRect (wxPoint (60, 40), wxSize (680, 460));
            break;

        default:
            if (scr_x < 800)    // just in case...
            {
                return wxRect (wxPoint (20, 10), wxSize (scr_x - 40, scr_y - 80));
            }
            else
            {
                return wxRect (wxPoint (0.125 * scr_x, 0.12 * scr_y), wxSize (0.75 * scr_x, 0.75 * scr_y));
            }
            break;
    }
}


// ***** Member functions to handle the menu events for the icon size settings:

void MainFrame::clear_icon_sizes ()
{
    GetMenuBar()->Check (ID_20pixels, false);
    GetMenuBar()->Check (ID_24pixels, false);
    GetMenuBar()->Check (ID_28pixels, false);
    GetMenuBar()->Check (ID_32pixels, false);
    GetMenuBar()->Check (ID_40pixels, false);
    GetMenuBar()->Check (ID_48pixels, false);
    GetMenuBar()->Check (ID_56pixels, false);
    GetMenuBar()->Check (ID_64pixels, false);
    GetMenuBar()->Check (ID_80pixels, false);
    GetMenuBar()->Check (ID_100pixels, false);
}

void MainFrame::save_icon_size (int size) const
{
    user_preferences.icon_size = size;
    write_config_file (translation.return_language());
}

void MainFrame::on_20pixels (wxCommandEvent &)
{
    clear_icon_sizes();
    GetMenuBar()->Check (ID_20pixels, true);
    save_icon_size (20);
}

void MainFrame::on_24pixels (wxCommandEvent &)
{
    clear_icon_sizes();
    GetMenuBar()->Check (ID_24pixels, true);
    save_icon_size (24);
}

void MainFrame::on_28pixels (wxCommandEvent &)
{
    clear_icon_sizes();
    GetMenuBar()->Check (ID_28pixels, true);
    save_icon_size (28);
}

void MainFrame::on_32pixels (wxCommandEvent &)
{
    clear_icon_sizes();
    GetMenuBar()->Check (ID_32pixels, true);
    save_icon_size (32);
}

void MainFrame::on_40pixels (wxCommandEvent &)
{
    clear_icon_sizes();
    GetMenuBar()->Check (ID_40pixels, true);
    save_icon_size (40);
}

void MainFrame::on_48pixels (wxCommandEvent &)
{
    clear_icon_sizes();
    GetMenuBar()->Check (ID_48pixels, true);
    save_icon_size (48);
}

void MainFrame::on_56pixels (wxCommandEvent &)
{
    clear_icon_sizes();
    GetMenuBar()->Check (ID_56pixels, true);
    save_icon_size (56);
}

void MainFrame::on_64pixels (wxCommandEvent &)
{
    clear_icon_sizes();
    GetMenuBar()->Check (ID_64pixels, true);
    save_icon_size (64);
}

void MainFrame::on_80pixels (wxCommandEvent &)
{
    clear_icon_sizes();
    GetMenuBar()->Check (ID_80pixels, true);
    save_icon_size (80);
}

void MainFrame::on_100pixels (wxCommandEvent &)
{
    clear_icon_sizes();
    GetMenuBar()->Check (ID_100pixels, true);
    save_icon_size (100);
}
