///////////////////////////////////////////////////////////////////////////////////////////
// Authors: Eva Bunodiere, Yasmin Kossak
// MainFrame.h
// MainFrame Class Declaration
// JPC36 Wx-View 
// July 25th 2000
// Technical Advisor : Carlos Moreno
///////////////////////////////////////////////////////////////////////////////////////////

#ifndef __MAIN_FRAME_H__
#define __MAIN_FRAME_H__

#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include "ZoomSettingsDialog.h"
#include "wx/timer.h"
#include "wx/image.h"
#include "wx/imaglist.h"
#include "wx/bitmap.h"
#include "wx/dc.h"
#include "wx/file.h"
#include "wx/stattext.h"
#include "wx/toolbar.h"
#include "wx/tbarbase.h"
#include "wx/msgdlg.h"
#include "wx/scrolwin.h"
#include "wx/dialog.h"
#include "wx/radiobox.h"
#include "wx/textctrl.h"
#include "wx/button.h"
#include "wx/event.h"
#include "slide_show.h"

#include <math.h>
#include <string>
#include <fstream>
#include <map>
using namespace std;


class Canvas;

class MainFrame: public wxFrame
{
public:
	MainFrame(const wxString& title, const wxPoint & pos, const wxSize & size);

	void on_properties(wxCommandEvent& WXUNUSED(event));
	void on_print(wxCommandEvent& WXUNUSED(event));
	void on_select_files_category(wxCommandEvent& WXUNUSED(event));
	void on_select_files_filename(wxCommandEvent& WXUNUSED(event));
	void on_exit(wxCommandEvent & WXUNUSED (event));
	void on_set_interval(wxCommandEvent& WXUNUSED(event));
	void on_show_files(wxCommandEvent& WXUNUSED(event));
	void on_run(wxCommandEvent& WXUNUSED(event));
	void on_rotate(wxCommandEvent& WXUNUSED(event));
	void on_horizontal_flip(wxCommandEvent& WXUNUSED(event));
	void on_vertical_flip(wxCommandEvent& WXUNUSED(event));
	void on_zoom_in(wxCommandEvent& WXUNUSED(event));
	void on_zoom_out(wxCommandEvent& WXUNUSED(event));
	void on_increase_contrast(wxCommandEvent& WXUNUSED(event));
	void on_decrease_contrast(wxCommandEvent& WXUNUSED(event));
	void on_increase_brightness(wxCommandEvent& WXUNUSED(event));
	void on_decrease_brightness(wxCommandEvent& WXUNUSED(event));
	void on_original_image( wxCommandEvent& WXUNUSED(event));
	void on_language(wxCommandEvent& WXUNUSED(event));
	void on_toggle_toolbar(wxCommandEvent & WXUNUSED(event));
//	void on_directory(wxCommandEvent& WXUNUSED(event));
	void on_contents(wxCommandEvent& WXUNUSED(event));
	void on_about(wxCommandEvent& WXUNUSED(event));
	void on_zoom_settings(wxCommandEvent& WXUNUSED(event));
	void write_config_file(const string & lang) const;
	void enable_image_items();
    void pop_up_menu(wxMouseEvent& event);
    void on_key_down(wxKeyEvent & event);

    void clear_icon_sizes ();
    void save_icon_size (int size) const;
    void on_20pixels (wxCommandEvent &);
    void on_24pixels (wxCommandEvent &);
    void on_28pixels (wxCommandEvent &);
    void on_32pixels (wxCommandEvent &);
    void on_40pixels (wxCommandEvent &);
    void on_48pixels (wxCommandEvent &);
    void on_56pixels (wxCommandEvent &);
    void on_64pixels (wxCommandEvent &);
    void on_80pixels (wxCommandEvent &);
    void on_100pixels (wxCommandEvent &);

private:

	wxToolBarBase *toolbar;
	wxMenu * popup_menu;
	
	string default_directory;
	int zoom_factor;
	int interval;
	bool show_toolbar;
	bool ZoomStyle;
	wxString icon_map;
    wxDialog * ZoomSettings;
	
	wxString file;
	Canvas * canvas;
	
	vector <wxString> image_listing;
	Slide_show * slide_show;
	
	bool image_open;
    int file_index;

	bool init_toolbar();
    void init_popup_menu ();
	
	DECLARE_EVENT_TABLE()
};

enum
{
	
	ID_main_frame_properties = 1,
    ID_main_frame_print,
    ID_main_frame_select_files,
	ID_main_frame_select_files_filename,
	ID_main_frame_select_files_category,
    ID_main_frame_reload,
	ID_main_frame_exit,
    ID_main_frame_set_interval,
    ID_main_frame_show_files,
    ID_main_frame_run,
    ID_main_frame_rotate,
    ID_main_frame_horizontal_flip,
    ID_main_frame_vertical_flip,
    ID_main_frame_zoom_in,
    ID_main_frame_zoom_out,
    ID_main_frame_increase_contrast,
    ID_main_frame_decrease_contrast,
    ID_main_frame_increase_brightness,
    ID_main_frame_decrease_brightness,
    ID_main_frame_original_image,
	ID_main_frame_language,
    ID_toggle_toolbar,
//    ID_main_frame_directory,
	ID_main_frame_zoom_settings,
    ID_main_frame_contents,
    ID_main_frame_about,
	ID_toolbar,
	ID_contents_frame,
	ID_canvas,
	
	ID_options_icon_size,
	ID_20pixels,
	ID_24pixels,
	ID_28pixels,
	ID_32pixels,
	ID_40pixels,
	ID_48pixels,
	ID_56pixels,
	ID_64pixels,
	ID_80pixels,
	ID_100pixels,
};

#endif		//!def __MAIN_FRAME_H__
